<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Controller;

use Eccube\Application;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Form\Type\ConvenienceType;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_CONVENIENCE;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Controller to handle module setting screen
 */
class ConvenienceController {

    public $app;
    public $const;

    // 【フロントページ】コンビニ決済：決済完了通知ページ Controller
    public function convPaymentResult(\Eccube\Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $rcode = '0' ;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // 開始
        $objMdl->printLog("コンビニ決済：決済完了(入金)通知処理開始");

        if(count($_REQUEST) == 0) {
            $objMdl->printLog("コンビニ決済：受信データ存在せず 決済完了(入金)通知処理終了");
            $rcode = '9';
        }

        if( $rcode == '0' ) {
            // パラメータ構築
            $arrReq = array();
            $request_msg = "コンビニ決済：決済完了(入金)通知処理 受信パラメータ\n" ;
            $i = 1;
            foreach($_REQUEST as $key=>$val){
                $arrReq[$key] = $val;
                $request_msg .= "　".$i.":".$key." ... ".$val."\n";
                $i++;
            }
            $objMdl->printLog($request_msg);

            // 受注拡張データ取得
            $PaydesignOrderInfo =
                $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'sid' => $_REQUEST["SID"]) );

            if( !is_null($PaydesignOrderInfo) ) {
                // 決済完了(入金)処理（受注ステータスを入金済みにする・入金日を設定する）
                $Order = $this->app['eccube.repository.order']->find($PaydesignOrderInfo->getOrderId());
                $order_status = $Order->getOrderStatus()->getId();
                if( $order_status == $this->app['config']['order_pay_wait'] ) {
                    // 『入金待ち』の受注ステータスでのみ決済完了(入金)処理を行う
                    $now = new \DateTime();
                    $Order
                        ->setPaymentDate($now)
                        ->setUpdateDate($now)
                        ->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_pre_end']));
                    $this->app['orm.em']->persist($Order);
                    $this->app['orm.em']->flush();
                } else {
                    // 『入金待ち』以外の受注ステータスではキャンセル不可
                    $order_status_name = $Order->getOrderStatus()->getName();
                    $objMdl->printLog("コンビニ決済：決済完了(入金)処理 受注ステータスエラー【現在の受注ステータス：".$order_status_name."】");
                    $rcode = '9';
                }
            } else {
                // データが取得できなかった場合はエラーログ出力
                $objMdl->printLog("コンビニ決済：決済完了(入金)通知処理 受注拡張データ取得・更新失敗");
                $rcode = '9';
            }
        }

        // セッション情報が残っている場合は削除
        if( !is_null($this->app['session']->get('eccube.plugin.paydesign.orderId')) ) {
            $this->app['session']->remove('eccube.plugin.paydesign.orderId'); }

        // 終了
        $objMdl->printLog("コンビニ決済：決済完了通知処理終了[".$rcode."]");
        if( $objMdl->checkEccubeOldVersion011() ) {
            // 3.0.10以前のバージョン
            header("Content-Type: text/plain; charset=Shift_JIS");
            echo $rcode;
            exit;
        } else {
            // 3.0.11以降のバージョン
            $response = new Response(
                'Content',
                Response::HTTP_OK,
                array( 'Content-Type' => 'text/plain')
            );
            $response->setCharset('Shift_JIS');
            $response->setContent($rcode);
            $response->prepare($this->app['request']);
            $response->send();
            return $response;
        }
    }

    // 【フロントページ】コンビニ決済：決済取消通知ページ Controller
    public function convCancelResult(\Eccube\Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
        $rcode = '0' ;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // 開始
        $objMdl->printLog("コンビニ決済：決済取消処理開始");

        if(count($_REQUEST) == 0) {
            $objMdl->printLog("コンビニ決済：受信データ存在せず 決済取消処理終了");
            $rcode = '9';
        }

        if( $rcode == '0' ) {
            // パラメータ構築
            $arrReq = array();
            $request_msg = "コンビニ決済：決済取消処理 受信パラメータ\n" ;
            $i = 1;
            foreach($_REQUEST as $key=>$val){
                $arrReq[$key] = $val;
                $request_msg .= "　".$i.":".$key." ... ".$val."\n";
                $i++;
            }
            $objMdl->printLog($request_msg);

            // 受注拡張データ取得
            $PaydesignOrderInfo =
                $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'sid' => $_REQUEST["SID"]) );

            if( !is_null($PaydesignOrderInfo) ) {
                // キャンセル処理（受注ステータスをキャンセルにする・入金日を初期化する）
                $Order = $this->app['eccube.repository.order']->find($PaydesignOrderInfo->getOrderId());
                $order_status = $Order->getOrderStatus()->getId();
                // 『入金済み(order_pre_end)』の受注ステータスでのみキャンセル処理を行う
                if( $order_status == $this->app['config']['order_pre_end'] ) {
                    $now = new \DateTime();
                    $Order
                        ->setPaymentDate(NULL)
                        ->setUpdateDate($now)
                        ->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_cancel']));
                    $this->app['orm.em']->persist($Order);
                    $this->app['orm.em']->flush();
                } else {
                    // 『入金済み』以外の受注ステータスではキャンセル不可
                    $order_status_name = $Order->getOrderStatus()->getName();
                    $objMdl->printLog("コンビニ決済：決済取消処理 受注ステータスエラー【現在の受注ステータス：".$order_status_name."】");
                    $rcode = '9';
                }
            } else {
                // データが取得できなかった場合はエラーログ出力
                $objMdl->printLog("コンビニ決済：決済取消処理 受注拡張データ取得・更新失敗");
                $rcode = '9';
            }
        }

        // 終了
        $objMdl->printLog("コンビニ決済：決済取消処理終了[".$rcode."]");
        if( $objMdl->checkEccubeOldVersion011() ) {
            // 3.0.10以前のバージョン
            header("Content-Type: text/plain; charset=Shift_JIS");
            echo $rcode;
            exit;
        } else {
            // 3.0.11以降のバージョン
            $response = new Response(
                'Content',
                Response::HTTP_OK,
                array( 'Content-Type' => 'text/plain')
            );
            $response->setCharset('Shift_JIS');
            $response->setContent($rcode);
            $response->prepare($this->app['request']);
            $response->send();
            return $response;
        }
    }

    // 【フロントページ】コンビニ決済ページ Controller
    public function index(\Eccube\Application $app)
    {
        $this->app = $app;
        $error_message = "" ;

        $Order = null;
        // 注文情報の確認
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        if (is_null($Order)) {
            $error_message = "注文情報の取得が出来ませんでした。この手続きは無効となりました。";
            return $app['view']->render('error.twig', array('error_message' => $error_message));
        }

        // プラグインユーティリティオブジェクトの作成
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // フォームに渡すデータの作成
        $convenienceForm = new ConvenienceType($this->app);
        $form = $this->app['form.factory']->createBuilder($convenienceForm)->getForm();

        if ('POST' === $this->app['request']->getMethod()) {
            $mode = $this->app['request']->request->get("mode");
            if ("next" == $mode) {
                $form->handleRequest($this->app['request']);
                if($form->isValid()){
                    // 選択したコンビニ情報を取得
                    $convenience = $form['convenience']->getData();
                    // データ送受信
                    $objClientConvenience = new PAYDESIGN_PLG_DCHK_CONVENIENCE($app);
                    $error_message = $objClientConvenience->execSettlement($Order, $convenience);
                    if( $error_message == "" ) {
                        // 正常処理終了時は注文完了ページに遷移
                        return $this->app->redirect($this->app['url_generator']->generate('paydesign_shopping_convenience_complete'));
                    }
                }
            }
        }

        return $app['view']->render('PayDesign/View/paydesign_convenience.twig', array(
             'form'  => $form->createView(),
             'title' => $objMdl->getPaymentConvenienceMethod(),
             'error_message' => $error_message,
        ));
    }

    // 【フロントページ】コンビニ決済ページ 『ご注文完了ページへ』ボタン処理 Controller
    public function complete(\Eccube\Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];

        // 注文情報の確認
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        // カート初期化(削除)
        $cartService = $this->app['eccube.service.cart'];
        $cartService->clear()->save();

        // 拡張受注情報を取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));

        // 完了ページ出力情報の作成
        $objClientConvenience = new PAYDESIGN_PLG_DCHK_CONVENIENCE($app);
        $ConvInfo = $objClientConvenience->getConvInfo($PaydesignOrderInfo);

        // セッション情報が残っている場合は削除
        if( !is_null($this->app['session']->get('eccube.plugin.paydesign.orderId')) ) {
            $this->app['session']->remove('eccube.plugin.paydesign.orderId'); }

        // 完了ページの表示
        return $app['view']->render('PayDesign/View/paydesign_convenience_complete.twig', array(
            'ConvInfo' => $ConvInfo,
        ));
    }

    // 【フロントページ】コンビニ決済ページ 『戻る』ボタン処理 Controller
    public function back(\Eccube\Application $app)
    {
        // 初期化
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];

        // 注文情報の確認
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }

        // 『戻る』場合はセッション情報を一度削除
        if( !is_null($this->app['session']->get('eccube.plugin.paydesign.orderId')) ) {
            $this->app['session']->remove('eccube.plugin.paydesign.orderId'); }

        // 受注情報を更新（購入処理中として更新する）
        if( !is_null($Order) ) {
            $Order->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_processing']));
            $this->app['orm.em']->persist($Order);
            $this->app['orm.em']->flush();
        }

        return $this->app->redirect($app->url('shopping'));
    }
}
?>
