<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Controller;

use Eccube\Application;
use Eccube\Entity\Master\CsvType;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Service\client\PAYDESIGN_PLG_DCHK_PERIODIC;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Controller to handle module setting screen
 */
class PeriodicController {

    // 定期情報一覧ページの表示
    public function index(Application $app, Request $request, $page_no = null)
    {
        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($app);
        $objMdl->install();
        $periodicStatus = $objMdl->getPeriodicStatus();

        $session = $request->getSession();

        $searchForm = $app['form.factory']
            ->createBuilder('admin_search_periodic')
            ->getForm();

        $pagination = array();

        $disps = $app['eccube.repository.master.disp']->findAll();
        $pageMaxis = $app['eccube.repository.master.page_max']->findAll();
        $page_count = $app['config']['default_page_count'];
        $page_status = null;
        $active = false;

        if ('POST' === $request->getMethod()) {

            $searchForm->handleRequest($request);
            $searchData = $searchForm->getData();

            if ($searchForm->isValid()) {
                // paginator
                $qb = $app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->getQueryBuilderBySearchDataForAdmin($searchData);
                $page_no = 1;
                $pagination = $app['paginator']()->paginate(
                    $qb,
                    $page_no,
                    $page_count
                );

            }

            // sessionのデータ保持
            $session->set('eccube.admin.periodic.search', $searchData);
            $active = true;

        } else {
            if (is_null($page_no)) {
                // sessionを削除
                $session->remove('eccube.admin.periodic.search');
            } else {
                // pagingなどの処理
                $searchData = $session->get('eccube.admin.periodic.search');
                if (!is_null($searchData)) {

                    // 公開ステータス
                    $status = $request->get('status');
                    if (!empty($status)) {
                        if ($status != $app['config']['admin_product_stock_status']) {
                            $searchData['status']->clear();
                            $searchData['status']->add($status);
                        } else {
                            $searchData['stock_status'] = $app['config']['disabled'];
                        }
                        $page_status = $status;
                    }
                    // 表示件数
                    $pcount = $request->get('page_count');

                    $page_count = empty($pcount) ? $page_count : $pcount;

                    $qb = $app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->getQueryBuilderBySearchDataForAdmin($searchData);
                    $pagination = $app['paginator']()->paginate(
                        $qb,
                        $page_no,
                        $page_count
                    );

                    // セッションから検索条件を復元
                    if (!empty($searchData['status'])) {
                        $searchData['status'] = $app['eccube.repository.master.order_status']->find($searchData['status']);
                    }
                    if (count($searchData['payment']) > 0) {
                        $payment_ids = array();
                        foreach ($searchData['payment'] as $Payment) {
                            $payment_ids[] = $Payment->getId();
                        }
                        $searchData['payment'] = $app['eccube.repository.payment']->findBy(array('id' => $payment_ids));
                    }
                    $searchForm->setData($searchData);
                    $active = true;
                }
            }
        }

        return $app->render('PayDesign/View/admin/periodic_index.twig', array(
            'searchForm' => $searchForm->createView(),
            'pagination' => $pagination,
            'disps' => $disps,
            'pageMaxis' => $pageMaxis,
            'page_no' => $page_no,
            'page_status' => $page_status,
            'page_count' => $page_count,
            'active' => $active,
            'periodicStatus' => $periodicStatus,
        ));

    }

    // 定期情報の編集
    public function edit(Application $app, Request $request, $id = null)
    {
        $TargetPeriodic = NULL;
        if (is_null($id)) {
            // 空のエンティティを作成.
            $TargetPeriodic = new \Plugin\PayDesign\Entity\PayDesignPeriodicInfo();
        } else {
            $TargetPeriodic = $app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->find($id);
            if (is_null($TargetPeriodic)) {
                throw new NotFoundHttpException();
            }
        }

        // 編集フォームにデータの設定
        $form = $this->setPeriodicEditForm($app, $TargetPeriodic);
        
        $err_message = "";
        if ('POST' === $request->getMethod()) {
            $form->handleRequest($request);
            // 登録ボタン押下
            switch ($request->get('mode')) {
                case 'register':
                    if($form->isValid()) {
                        // 定期マスター編集フォームを基にした定期マスターの更新
                        $err_message = $this->updatePeriodicInfo($app, $TargetPeriodic, $form);
                        if( $err_message == "" ) {
                            $app->addSuccess('定期マスターの更新が完了しました。', 'admin');
                        } else { 
                            $app->addError($err_message,'admin');
                        }
                    }
                    break;
                default:
                    break;
            }
        }

        return $app->render('PayDesign/View/admin/periodic_edit.twig', array(
            'form' => $form->createView(),
            'Periodic' => $TargetPeriodic,
            'id' => $id,
        ));
    }

    // 定期情報の削除
    public function delete(Application $app, $id)
    {
        $Periodic = $app['orm.em']->getRepository('Plugin\PayDesign\Entity\PayDesignPeriodicInfo')
            ->find($id);

        if ($Periodic) {
            $now = new \DateTime();
            $Periodic
                ->setUpdateDate($now)
                ->setDelFlg(1);
            $app['orm.em']->persist($Periodic);
            $app['orm.em']->flush();

            $app->addSuccess('定期情報を削除しました。', 'admin');
        }

        return $app->redirect($app->url('plugin_PayDesign_periodic'));
    }

    // 定期情報CSVの出力
    public function exportCsv(Application $app, Request $request)
    {
        // タイムアウトを無効にする.
        set_time_limit(0);

        // Localeの設定
        setlocale(LC_ALL, 'ja_JP.UTF-8');

        // sql loggerを無効にする.
        $em = $app['orm.em'];
        $em->getConfiguration()->setSQLLogger(null);

        // プラグイン設定の取得
        $objMdl =& PluginUtil::getInstance($app);
        $objMdl->install();
        $periodicStatus = $objMdl->getPeriodicStatus();

        // 都道府県データ
        $Pref = array();
        $PrefAll = $app['eccube.repository.master.pref']->findAll();
        foreach( $PrefAll as $pref ) { $Pref[$pref->getId()] = $pref->getName(); }

        // 出力用のベースデータの取得
        $searchForm = $app['form.factory']
            ->createBuilder('admin_search_periodic')
            ->getForm();
        $searchData = $searchForm->getData();
        $qb = $app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->getQueryBuilderBySearchDataForAdmin($searchData);
        $Periodic = $qb->getQuery()->getResult();

        // 出力データ作成
        $response = $app->render('PayDesign/View/csv/periodic_master.twig', array(
            'PeriodicInfos' => $Periodic,
            'PeriodicStatus' => $periodicStatus,
            'Pref' => $Pref,
        ));
        // エンコードの種類を明確に定義(UTF-8)する
      //$contents = mb_convert_encoding($response->getContent(), 'SJIS-win', mb_internal_encoding());
        $contents = mb_convert_encoding($response->getContent(), 'SJIS-win', 'UTF-8');

        // ファイルとして出力
        $now = new \DateTime();
        $filename = 'periodic_' . $now->format('YmdHis') . '.csv';
        $response->setContent($contents);
        $response->headers->set('Content-Type', 'application/octet-stream');
        $response->headers->set('Content-Disposition', 'attachment; filename=' . $filename);

        return $response;
    }

    // 定期注文情報作成・対象者表示ページ
    public function create_index(Application $app, Request $request)
    {
        $err_message = "";

        $searchForm = $app['form.factory']
            ->createBuilder('admin_search_periodic_create')
            ->getForm();

        $CreateResult  = NULL;
        $PeriodicInfos = NULL;
        $is_search = false;
        $is_create = false ;
        if ('POST' === $request->getMethod()) {
            $searchForm->handleRequest($request);
            $searchData = $searchForm->getData();
            if ($searchForm->isValid()) {
                switch($request->get('mode')) {
                    case 'periodic_search':
                        // 検索＆表示のみ
                        if( $searchData['periodic_next_date_start'] <= $searchData['periodic_next_date_end'] ) {
                            // 検索結果
                            $qb = $app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->getQueryBuilderBySearchCreateDataForAdmin($searchData);
                            $PeriodicInfos = $qb->getQuery()->getResult();
                         } else {
                             $err_message = "次回注文日の開始と終了の条件が不正です。設定をご確認ください。";
                         }
                         $is_search = true;
                         break;
                     case 'periodic_create':
                         // 検索＆定期注文作成
                         if( $searchData['periodic_next_date_start'] <= $searchData['periodic_next_date_end'] ) {
                            // 検索結果
                            $qb = $app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->getQueryBuilderBySearchCreateDataForAdmin($searchData);
                            $PeriodicInfos = $qb->getQuery()->getResult();
                            // 定期マスタ作成処理の実行
                            if( $PeriodicInfos ) {
                                $objClientPeriodic = new PAYDESIGN_PLG_DCHK_PERIODIC($app);
                                $CreateResult = $objClientPeriodic->makePeriodicOrder($PeriodicInfos);
                            }
                         } else {
                             $err_message = "次回注文日の開始と終了の条件が不正です。設定をご確認ください。";
                         }
                         $is_search = true;
                         $is_create = true;
                         break;
                     default:
                         break;
                 }
            }
        } else {
            // 初期検索値として、現在の日付を設定しておく
            $now = new \DateTime();
            $searchForm->get('periodic_next_date_start')->setData($now);
            $searchForm->get('periodic_next_date_end')->setData($now);
        }

        return $app->render('PayDesign/View/admin/periodic_create.twig', array(
            'searchForm'    => $searchForm->createView(),
            'PeriodicInfos' => $PeriodicInfos,
            'CreateResult'  => $CreateResult,
            'err_message'   => $err_message,
            'is_search'    => $is_search,
            'is_create'    => $is_create,
        ));
    }

    // 【バッチ運用】定期注文の作成
    public function PeriodicMakeOrder(Application $app)
    {
        $objMdl =& PluginUtil::getInstance($app);
        $objMdl->install();

        $objMdl->printLog("【バッチ】定期注文作成開始");

        // Cron（シェル）からの実行でない場合は、何もしない
        if( isset($_SERVER['SHELL']) ) {
            // 検索条件設定
            $now = new \DateTime();
            $searchData = array();
            $searchData['periodic_next_date_start'] = new \DateTime($now->format('Y-m-d 00:00:00'));
            $searchData['periodic_next_date_end']   = new \DateTime($now->format('Y-m-d 23:59:59'));
            $objMdl->printLog("検索期間：".$searchData['periodic_next_date_start']->format('Y-m-d H:i:s')." ～ ".$searchData['periodic_next_date_end']->format('Y-m-d H:i:s'));

            // 検索結果の取得
            $qb = $app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->getQueryBuilderBySearchCreateDataForAdmin($searchData);
            $PeriodicInfos = $qb->getQuery()->getResult();

            // 定期マスタ作成処理の実行
            if( $PeriodicInfos ) {
                $objClientPeriodic = new PAYDESIGN_PLG_DCHK_PERIODIC($app);
                $CreateResult = $objClientPeriodic->makePeriodicOrder($PeriodicInfos);
            } else {
                $objMdl->printLog("【バッチ】定期注文作成：対象データはありませんでした。");
            }

        } else {
            $objMdl->printLog("【バッチ】定期注文作成：【実行エラー】ブラウザからの不正アクセス確認");
        }

        $objMdl->printLog("【バッチ】定期注文作成終了");

        return $app->render('PayDesign/View/batch/periodic_make_order_result.twig',array());
    }

    // 定期マスター編集フォームにデータの設定
    private function setPeriodicEditForm($app, $TargetPeriodic)
    {
        // ベースのデータ作成
        $form = NULL;
        $form = $app['form.factory']
            ->createBuilder('admin_periodic_info_edit')
            ->getForm();

        // データ設定
        $form->get('periodic_status')->setData($TargetPeriodic->getPeriodicStatus());
        $form->get('periodic_next_date')->setData($TargetPeriodic->getPeriodicNextDate());
        $form->get('periodic_interval')->setData($TargetPeriodic->getPeriodicInterval());
        $form->get('periodic_create_day')->setData($TargetPeriodic->getPeriodicCreateDay());
        $form->get('price')->setData($TargetPeriodic->getPrice());
        $form->get('quantity')->setData($TargetPeriodic->getQuantity());
        $form->get('name01')->setData($TargetPeriodic->getShippingName01());
        $form->get('name02')->setData($TargetPeriodic->getShippingName02());
        $form->get('kana01')->setData($TargetPeriodic->getShippingKana01());
        $form->get('kana02')->setData($TargetPeriodic->getShippingKana02());
        $form->get('pref')->setData($TargetPeriodic->getShippingPref());
        $form->get('zip01')->setData($TargetPeriodic->getShippingZip01());
        $form->get('zip02')->setData($TargetPeriodic->getShippingZip02());
        $form->get('addr01')->setData($TargetPeriodic->getShippingAddr01());
        $form->get('addr02')->setData($TargetPeriodic->getShippingAddr02());
        $form->get('tel01')->setData($TargetPeriodic->getShippingTel01());
        $form->get('tel02')->setData($TargetPeriodic->getShippingTel02());
        $form->get('tel03')->setData($TargetPeriodic->getShippingTel03());
        $form->get('fax01')->setData($TargetPeriodic->getShippingFax01());
        $form->get('fax02')->setData($TargetPeriodic->getShippingFax02());
        $form->get('fax03')->setData($TargetPeriodic->getShippingFax03());

        return $form;
    }

    // 定期マスター編集フォームを基にした定期マスターの更新
    private function updatePeriodicInfo($app, $TargetPeriodic, $form) {

        // 初期化
        $err_message = "";
        $PeriodicInfo = $TargetPeriodic;
        $now = new \DateTime();
        $data = $form->getData();

        // 退会済み会員かどうか
        $customer_flg = true;
        $Customer = $PeriodicInfo->getCustomer();
        if(is_null($Customer)) { $customer_flg = false; }

        if( $now > $data['periodic_next_date'] ) {
            $err_message = "次回注文日は未来の日付を設定してください。";
        } 

        if(!is_null($Customer) && $data['periodic_status'] == 4) {
            $err_message = "退会状態に無い会員の申込状況を【退会】にする事はできません。";
        }

        if(is_null($Customer) && $data['periodic_status'] != 4) {
            $err_message = "退会会員の申込状況を【退会】以外にする事はできません。";
        }

        if( $err_message == "" ) {
            // 入力されたデータを更新する
            $PeriodicInfo->setPeriodicStatus($data['periodic_status']);
            $PeriodicInfo->setPeriodicNextDate($data['periodic_next_date']);
            $PeriodicInfo->setPeriodicInterval($data['periodic_interval']);
            $PeriodicInfo->setPeriodicCreateDay($data['periodic_create_day']);
            $PeriodicInfo->setPrice($data['price']);
            $PeriodicInfo->setQuantity($data['quantity']);
            $PeriodicInfo->setShippingName01($data['name01']);
            $PeriodicInfo->setShippingName02($data['name02']);
            $PeriodicInfo->setShippingKana01($data['kana01']);
            $PeriodicInfo->setShippingKana02($data['kana02']);
            $PeriodicInfo->setShippingPref($data['pref']);
            $PeriodicInfo->setShippingZip01($data['zip01']);
            $PeriodicInfo->setShippingZip02($data['zip02']);
            $PeriodicInfo->setShippingZipCode($data['zip01'].$data['zip02']);
            $PeriodicInfo->setShippingAddr01($data['addr01']);
            $PeriodicInfo->setShippingAddr02($data['addr02']);
            $PeriodicInfo->setShippingTel01($data['tel01']);
            $PeriodicInfo->setShippingTel02($data['tel02']);
            $PeriodicInfo->setShippingTel03($data['tel03']);
            $PeriodicInfo->setShippingFax01($data['fax01']);
            $PeriodicInfo->setShippingFax02($data['fax02']);
            $PeriodicInfo->setShippingFax03($data['fax03']);
            $app['orm.em']->persist($PeriodicInfo);
            $app['orm.em']->flush();
        }

        return $err_message ;
    }
}
?>
