<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Form\Type;

use Plugin\PayDesign\Controller\Util\CommonUtil;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Validator\Constraints as Assert;

class AdminProductPeriodicInfoType extends AbstractType
{
    private $app;

    public function __construct(\Eccube\Application $app)
    {
        $this->app = $app;
    }

    /**
     * Build config type form
     *
     * @param FormBuilderInterface $builder
     * @param array $options
     * @return type
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $objPluginUtil =& PluginUtil::getInstance($this->app);
        $objPluginUtil->install();
        $arrPeriodicSettleMethod = $objPluginUtil->getPeriodicSettleMethod();
        $arrPeriodicSettleMethod[1] .= "　"; // 【暫定処理】フォームの表示間隔の調整

        $objCommonUtil = new CommonUtil();
        $arrPeriodicInterval = $objCommonUtil->getMonthSelectParameter('ヶ月');
        $arrCutoffDay = $objCommonUtil->getDaySelectParameter('日');
        $arrCreateDay = $objCommonUtil->getDaySelectParameter('日');

        $builder
            ->add('periodic_flg', 'choice', array(
                'label' => '定期商品フラグ',
                'choices' => array('1' => 'この商品を定期商品として販売する'),
                'multiple' => true,
                'expanded' => true,
                'required' => true,
            ))
            ->add('periodic_price', 'text', array(
                'label' => '定期販売時価格',
                'attr' => array(
                    'class' => 'lockon_card_row',
                ),
                'constraints' => array(
                    new Assert\Regex(array(
                        'pattern' => "/^\d+$/u",
                        'message' => '定期販売時価格は数字を設定してください。',
                    )),
                ),
            ))
            ->add('periodic_interval', 'choice', array(
                'label' => '購入間隔',
                'choices' => $arrPeriodicInterval,
                'multiple'=> false,
                'expanded' => false,
                'required' => false,
                'empty_value' => '購入間隔を選択してください',
                'empty_data' => '',
            ))
            ->add('periodic_cutoff_day', 'choice', array(
                'label' => '申込締切日',
                'choices' => $arrCutoffDay,
                'multiple'=> false,
                'expanded' => false,
                'required' => false,
                'empty_value' => '申込締切日を選択してください',
                'empty_data' => '',
            ))
            ->add('periodic_create_day', 'choice', array(
                'label' => '次回注文基準日',
                'choices' => $arrCreateDay,
                'multiple'=> false,
                'expanded' => false,
                'required' => false,
                'empty_value' => '次回注文基準日を選択してください',
                'empty_data' => '',
            ))
            ->add('periodic_settle_method', 'choice', array(
                'label' => '申込時決済方法',
                'choices' => $arrPeriodicSettleMethod,
                'expanded' => true,
            ))
            // 複合Validation処理
            ->addEventListener(FormEvents::POST_SUBMIT, function ($event) {
                $form = $event->getForm();
                $data = $form->getData();
                $periodic_cutoff_day = $data['periodic_cutoff_day'];
                $periodic_create_day = $data['periodic_create_day'];
                if(!is_null($periodic_cutoff_day) && !is_null($periodic_create_day)) {
                    if($periodic_cutoff_day == $periodic_create_day) {
                        $form['periodic_cutoff_day']
                            ->addError(new FormError('申込締切日と次回注文基準日に同じ日付を設定する事は出来ません。'));
                    }
                    if($periodic_cutoff_day>27 && $periodic_create_day>$periodic_cutoff_day) {
                        $form['periodic_create_day']
                            ->addError(new FormError('申込締切日が28日以降の場合、次回注文基準日に申込締切日より大きな日付を設定することは出来ません。'));
                    }
                }
            })
            ->addEventSubscriber(new \Eccube\Event\FormEventSubscriber());
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'admin_product_periodic_info';
    }
}
?>
