<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Service\client;

require_once(__DIR__ . "/../../vendor/autoload.php");

use Plugin\PayDesign\Controller\Util\PluginUtil;

/**
 * 決済モジュール 決済処理: コンビニ決済処理
 */
class PAYDESIGN_PLG_DCHK_CONVENIENCE extends PAYDESIGN_PLG_DCHK {

    protected $app;
    protected $const;

    /**
     * コンストラクタ
     *
     * @return void
     */
    function __construct(\Eccube\Application $app)
    {
        parent::__construct($app);
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
    }

    /*
     * コンビニエンス決済要求実行
     */
    public function execSettlement($Order, $convenience) {

        // 初期化
        $err_message = "" ;
        $sendData = array();
        $get_limitdate = true;

        // プレフィックスワードの取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('コンビニ決済：決済要求実行 Start');

        // 送信データ作成

        // 基本情報
        $order_id = $Order->getId(); // 設定されている受注ID
        $sid = $userSettings['prefix_word'].$order_id;
        $sendData = $this->getBasicSendData($sid, 'convenience');

        // コンビニ情報
        $sendData = array_merge($sendData, $this->getConvenienceSendData($convenience, $get_limitdate));

        // 注文情報
        $sendData = array_merge($sendData, $this->getOrderSendData($Order));

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CONV_SEND_URL'], $sendData);

        // エラーがなければレスポンスデータ解析
        if( is_array($response_body) ) {
            // レスポンスを解析
            if( $response_body[1] == $sid ) {
                switch($response_body[0]) {
                    case $this->const['PDP_PAYMENT_RESPONSE_CODE_OK']:
                        // OK ... 正常処理（注文完了）
                        // ログ
                        $objMdl->printLog("コンビニ決済 注文完了処理（SID=".$sid."）");
                        // 拡張受注情報を取得
                        $PaydesignOrderInfo =
                            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));
                        if( !is_null($PaydesignOrderInfo) ) {
                            // 拡張注文情報を更新
                            $PaydesignOrderInfo
                                ->setSelectConvenienceKind($convenience)
                                ->setResultConvPaymentInfo($response_body[3])
                                ->setResultConvPaymentLimit($response_body[4])
                                ->setResultConvConfirmUrl($response_body[6]);
                            $this->app['orm.em']->persist($PaydesignOrderInfo);
                            $this->app['orm.em']->flush();
                            // 注文用メールテンプレートを取得
                            $MailTemplate = $this->app['eccube.repository.mail_template']->find(1);
                            // 店舗基本情報取得
                            $BaseInfo = $this->app['eccube.repository.base_info']->get();
                            // メール送信用データ作成
                            $tpl_path = "PayDesign/View/mail_template/convenience_order.twig";
                            $subject = '[' . $BaseInfo->getShopName() . '] ' . $MailTemplate->getSubject();
                            $to_mailaddr = $Order->getEmail();
                            $ConvInfo = $this->getConvInfo($PaydesignOrderInfo);
                            $body = $this->app->renderView("PayDesign/View/mail_template/convenience_order.twig", array(
                                'header'             => $MailTemplate->getHeader(),
                                'footer'             => $MailTemplate->getFooter(),
                                'Order'              => $Order,
                                'ConvInfo'           => $ConvInfo,
                            ));
                            // メール送信
                            $objMdl->sendMail( $subject, $to_mailaddr, $body );
                            // 注文情報の終了処理
                            $this->changeOrderData($Order);
                        } else {
                            // 拡張注文情報取得エラー
                            $err_message = '【注文エラー】注文情報の取得にしっぱいしました。お手数ですが店舗担当者にご確認をお願い致します。（'.$sid.'）';
                        }
                        break;
                    case $this->const['PDP_PAYMENT_RESPONSE_CODE_ERR']:
                    case $this->const['PDP_PAYMENT_RESPONSE_CODE_PRERR']:
                        // 異常終了
                        $err_message  = "【エラー】".$response_body[2]."<br />\n" ;
                        $err_message .= "エラーが発生しております。上記内容をご確認ください。";
                        break;
                    case $this->const['PDP_PAYMENT_RESPONSE_CODE_CVSERR']:
                        // コンビニ側システムエラー
                        $err_message = '【コンビニシステムエラー】選択したコンビニ側でエラーが発生いたしました。お手数ですが別のコンビニを選択していただくか、店舗担当者にご確認をお願い致します。' ;
                        break;
                    default:
                        break;
                }
            } else {
                $err_message = '【注文ID不一致】正しい結果を取得する事が出来ませんでした。お手数ですが店舗担当者にご確認をお願い致します。（'.$sid.'/'.$response_body[1].'）';
                $objMdl->printLog($err_message);
            }
        } else {
            $err_message = '【不正エラー】不正なレスポンスを取得しました。お手数ですが店舗担当者にご確認をお願い致します。' ;
            $objMdl->printLog($err_message);
        }

        // End
        $objMdl->printLog('コンビニ決済：決済要求実行 End');

        // 終了
        return $err_message;
    }

    /*
     * コンビニエンス決済：取消(返品)実行処理
     */
    public function cancelOrder($Order) {

        // 初期化
        $ret = true;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('コンビニ決済：申込キャンセル要求実行 Start');

        // 拡張注文情報の取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));

        // 送信データの作成(true は PASSも付加)
        $sendData = $this->getBasicSendData($PaydesignOrderInfo->getSid(), 'convenience', true);
        // 'FUKA'は不要なので削除
        unset($sendData['FUKA']);

        // コンビニ情報
        $convenience = $PaydesignOrderInfo->getSelectConvenienceKind();
        $sendData = array_merge($sendData, $this->getConvenienceSendData($convenience));

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CONV_CANCEL_URL'], $sendData);

        if( is_array($response_body) ) {
            // レスポンスを解析
            if( strstr($response_body[0],$PaydesignOrderInfo->getSid()) ) {
                // エラーメッセージの作成
                if( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_NG']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【申込キャンセルエラー】申込キャンセルに失敗しました。注文情報または決済管理画面をご確認ください。';
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_ERR']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【エラー】正しい結果が取得できませんでした。注文情報または決済管理画面をご確認ください。';
                }
            } else {
                $msg = '【SID不一致】正しい結果を取得する事が出来ませんでした。注文情報をご確認ください。';
                $this->setError($msg);
                $ret = false ;
            }
        } else {
            $ret = false ;
        }

        // End
        $objMdl->printLog('コンビニ決済：申込キャンセル要求実行 End');

        return $ret;
    }

    /*
     * コンビニエンス決済：コンビニエンス決済メール送信情報作成
     */
    public function getConvInfo( $PaydesignOrderInfo ) {

        // 初期化
        $ConvInfo = array();

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // コンビニ情報を取得
        $convenienceInfo = $objMdl->getConvenienceInfo();

        // 情報を設定
        $ConvInfo['conv_num'] = $PaydesignOrderInfo->getSelectConvenienceKind();
        $ConvInfo['conv_name'] = $convenienceInfo[$PaydesignOrderInfo->getSelectConvenienceKind()];
        $ConvInfo['conv_payment_info'] = $PaydesignOrderInfo->getResultConvPaymentInfo();
        $limit_date = $PaydesignOrderInfo->getResultConvPaymentLimit();
        $ConvInfo['conv_payment_limit'] = substr($limit_date,0,4)."/".substr($limit_date,4,2)."/".substr($limit_date,-2);
        $ConvInfo['conv_confirm_url'] = $PaydesignOrderInfo->getResultConvConfirmUrl();

        return $ConvInfo;
    }

    /*
     * 【送信用】コンビニエンス決済専用パラメータ設定
     */
    function getConvenienceSendData( $convenience, $get_limitdate = NULL) {

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // コンビニエンス種別指定
        $arrRet = array() ;
        $arrRet = array( 'STORE' => $convenience );

        // 必要があれば、支払期限の設定
        if( $get_limitdate ) {
            // 支払期限の計算
            $limitdate = date("Ymd",( time() + 86400 * $userSettings['convenience_limitdate']));

            // ログ
            $msg  = "コンビニ決済 専用パラメータ：\n" ;
            $msg .= "　1.STORE   ... ".$convenience."\n";
            $msg .= "　2.KIGEN   ... ".$limitdate;
            $objMdl->printLog($msg);

            // 支払期限を追加
            $arrRet['KIGEN'] = $limitdate;
        } else {
            // ログ
            $msg  = "コンビニ決済 専用パラメータ：\n" ;
            $msg .= "　1.STORE   ... ".$convenience."\n";
            $objMdl->printLog($msg);
        }

        return $arrRet ;
    }
}
?>
